@@ -1,6 +1,7 @@
 from pypy.interpreter.error import oefmt
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.typedef import TypeDef, interp_attrproperty
+from pypy.interpreter.typedef import GetSetProperty
 from pypy.interpreter.gateway import interp2app
 from rpython.rlib import jit
 
@@ -24,9 +25,8 @@ class W_FunctionWrapper(W_Root):
     This class cannot be used for variadic functions.
     """
     _immutable_ = True
-    common_doc_str = 'direct call to the C function of the same name'
 
-    def __init__(self, space, fnptr, directfnptr,
+    def __init__(self, space, ffi, fnptr, directfnptr,
                  rawfunctype, fnname, modulename):
         # everything related to the type of the function is accessed
         # as immutable attributes of the 'rawfunctype' object, which
@@ -39,6 +39,7 @@ class W_FunctionWrapper(W_Root):
         assert locs is None or len(ctype.fargs) == len(locs)
         #
         self.space = space
+        self.ffi = ffi
         self.fnptr = fnptr
         self.directfnptr = directfnptr
         self.rawfunctype = rawfunctype
@@ -93,6 +94,11 @@ class W_FunctionWrapper(W_Root):
     def descr_repr(self, space):
         return space.wrap("<FFIFunctionWrapper for %s()>" % (self.fnname,))
 
+    def descr_get_doc(self, space):
+        doc = self.rawfunctype.repr_fn_type(self.ffi, self.fnname)
+        doc = '%s;\n\nCFFI C function from %s.lib' % (doc, self.modulename)
+        return space.wrap(doc)
+
 
 @jit.unroll_safe
 def prepare_args(space, rawfunctype, args_w, start_index):
@@ -128,6 +134,6 @@ W_FunctionWrapper.typedef = TypeDef(
         __call__ = interp2app(W_FunctionWrapper.descr_call),
         __name__ = interp_attrproperty('fnname', cls=W_FunctionWrapper),
         __module__ = interp_attrproperty('modulename', cls=W_FunctionWrapper),
-        __doc__ = interp_attrproperty('common_doc_str', cls=W_FunctionWrapper),
+        __doc__ = GetSetProperty(W_FunctionWrapper.descr_get_doc),
         )
 W_FunctionWrapper.typedef.acceptable_as_base_class = False
