@@ -281,7 +281,8 @@ class AppTestNDArray(AppTestCpythonExtensionBase):
                     return _PyArray_DescrFromType(typenum);
                 """
                 ),
-                ], prologue='#include <numpy/arrayobject.h>')
+                ], prologue='''#include <numpy/arrayobject.h>
+#include <pypy_numpy.h>''')
         arr = mod.test_simplenew()
         assert arr.shape == (2, 3)
         assert arr.dtype.num == 11 #float32 dtype
@@ -309,7 +310,8 @@ class AppTestNDArray(AppTestCpythonExtensionBase):
                     Py_INCREF(obj);
                     return obj;
                 '''),
-                ], prologue='#include <numpy/arrayobject.h>')
+                ], prologue='''#include <numpy/arrayobject.h>
+#include <pypy_numpy.h>''')
         array = ndarray((3, 4), dtype='d')
         assert mod.check_array(array) is array
         raises(TypeError, "mod.check_array(42)")
@@ -353,6 +355,7 @@ class AppTestNDArray(AppTestCpythonExtensionBase):
                 """),
                 ], prologue='''
                 #include "numpy/ndarraytypes.h"
+                #include "pypy_numpy.h"
                 /*#include <numpy/ufuncobject.h> generated by numpy setup.py*/
                 typedef void (*PyUFuncGenericFunction)
                             (char **args,
