@@ -8,6 +8,7 @@ from pypy.interpreter.baseobjspace import InternalSpaceCache, W_Root, ObjSpace
 from pypy.interpreter.error import OperationError
 from rpython.rlib.objectmodel import specialize, instantiate
 from rpython.rlib.nonconst import NonConstant
+from rpython.rlib.rarithmetic import base_int
 from pypy.module.micronumpy import boxes, ufuncs
 from pypy.module.micronumpy.arrayops import where
 from pypy.module.micronumpy.ndarray import W_NDimArray
@@ -65,6 +66,7 @@ class FakeSpace(ObjSpace):
     w_KeyError = W_TypeObject("KeyError")
     w_SystemExit = W_TypeObject("SystemExit")
     w_KeyboardInterrupt = W_TypeObject("KeyboardInterrupt")
+    w_VisibleDeprecationWarning = W_TypeObject("VisibleDeprecationWarning")
     w_None = None
 
     w_bool = W_TypeObject("bool")
@@ -177,7 +179,7 @@ class FakeSpace(ObjSpace):
             return BoolObject(obj)
         elif isinstance(obj, int):
             return IntObject(obj)
-        elif isinstance(obj, long):
+        elif isinstance(obj, base_int):
             return LongObject(obj)
         elif isinstance(obj, W_Root):
             return obj
@@ -194,32 +196,36 @@ class FakeSpace(ObjSpace):
     def newfloat(self, f):
         return self.float(f)
 
+    def newslice(self, start, stop, step):
+        return SliceObject(self.int_w(start), self.int_w(stop),
+                           self.int_w(step))
+
     def le(self, w_obj1, w_obj2):
-        assert isinstance(w_obj1, boxes.W_GenericBox) 
-        assert isinstance(w_obj2, boxes.W_GenericBox) 
+        assert isinstance(w_obj1, boxes.W_GenericBox)
+        assert isinstance(w_obj2, boxes.W_GenericBox)
         return w_obj1.descr_le(self, w_obj2)
 
     def lt(self, w_obj1, w_obj2):
-        assert isinstance(w_obj1, boxes.W_GenericBox) 
-        assert isinstance(w_obj2, boxes.W_GenericBox) 
+        assert isinstance(w_obj1, boxes.W_GenericBox)
+        assert isinstance(w_obj2, boxes.W_GenericBox)
         return w_obj1.descr_lt(self, w_obj2)
 
     def ge(self, w_obj1, w_obj2):
-        assert isinstance(w_obj1, boxes.W_GenericBox) 
-        assert isinstance(w_obj2, boxes.W_GenericBox) 
+        assert isinstance(w_obj1, boxes.W_GenericBox)
+        assert isinstance(w_obj2, boxes.W_GenericBox)
         return w_obj1.descr_ge(self, w_obj2)
 
     def add(self, w_obj1, w_obj2):
-        assert isinstance(w_obj1, boxes.W_GenericBox) 
-        assert isinstance(w_obj2, boxes.W_GenericBox) 
+        assert isinstance(w_obj1, boxes.W_GenericBox)
+        assert isinstance(w_obj2, boxes.W_GenericBox)
         return w_obj1.descr_add(self, w_obj2)
 
     def sub(self, w_obj1, w_obj2):
         return self.wrap(1)
 
     def mul(self, w_obj1, w_obj2):
-        assert isinstance(w_obj1, boxes.W_GenericBox) 
-        assert isinstance(w_obj2, boxes.W_GenericBox) 
+        assert isinstance(w_obj1, boxes.W_GenericBox)
+        assert isinstance(w_obj2, boxes.W_GenericBox)
         return w_obj1.descr_mul(self, w_obj2)
 
     def pow(self, w_obj1, w_obj2, _):
@@ -402,6 +408,9 @@ class FakeSpace(ObjSpace):
         assert isinstance(w_check_class, W_TypeObject)
         return w_exc_type.name == w_check_class.name
 
+    def warn(self, w_msg, w_warn_type):
+        pass
+
 class FloatObject(W_Root):
     tp = FakeSpace.w_float
     def __init__(self, floatval):
@@ -832,7 +841,7 @@ class FunctionCall(Node):
             elif self.name == 'reshape':
                 w_arg = self.args[1]
                 assert isinstance(w_arg, ArrayConstant)
-                order = -1 
+                order = -1
                 w_res = arr.reshape(interp.space, w_arg.wrap(interp.space), order)
             else:
                 assert False
