@@ -32,11 +32,11 @@ from __future__ import absolute_import
 import inspect
 import weakref
 from types import BuiltinFunctionType, MethodType
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 
 import rpython
 from rpython.tool import descriptor
-from rpython.tool.pairtype import pair, extendabletype
+from rpython.tool.pairtype import pair, extendabletype, doubledispatch
 from rpython.rlib.rarithmetic import r_uint, base_int, r_singlefloat, r_longfloat
 
 
@@ -129,6 +129,16 @@ class SomeObject(object):
     def nonnoneify(self):
         return self
 
+@doubledispatch
+def intersection(s_obj1, s_obj2):
+    """Return the intersection of two annotations, or an over-approximation thereof"""
+    raise NotImplementedError
+
+@doubledispatch
+def difference(s_obj1, s_obj2):
+    """Return the set difference of two annotations, or an over-approximation thereof"""
+    raise NotImplementedError
+
 
 class SomeType(SomeObject):
     "Stands for a type.  We might not be sure which one it is."
@@ -138,6 +148,23 @@ class SomeType(SomeObject):
     def can_be_none(self):
         return False
 
+class SomeTypeOf(SomeType):
+    """The type of a variable"""
+    def __init__(self, args_v):
+        self.is_type_of = args_v
+
+def typeof(args_v):
+    if args_v:
+        result = SomeTypeOf(args_v)
+        if len(args_v) == 1:
+            s_arg = args_v[0].annotation
+            if isinstance(s_arg, SomeException) and len(s_arg.classdefs) == 1:
+                cdef, = s_arg.classdefs
+                result.const = cdef.classdesc.pyobj
+        return result
+    else:
+        return SomeType()
+
 
 class SomeFloat(SomeObject):
     "Stands for a float or an integer."
@@ -214,6 +241,9 @@ class SomeBool(SomeInteger):
 
     def set_knowntypedata(self, knowntypedata):
         assert not hasattr(self, 'knowntypedata')
+        for key, value in knowntypedata.items():
+            if not value:
+                del knowntypedata[key]
         if knowntypedata:
             self.knowntypedata = knowntypedata
 
@@ -409,7 +439,7 @@ class SomeInstance(SomeObject):
 
     def __init__(self, classdef, can_be_None=False, flags={}):
         self.classdef = classdef
-        self.knowntype = classdef or object
+        self.knowntype = classdef.classdesc if classdef else None
         self.can_be_None = can_be_None
         self.flags = flags
 
@@ -437,6 +467,52 @@ class SomeInstance(SomeObject):
     def noneify(self):
         return SomeInstance(self.classdef, can_be_None=True)
 
+@intersection.register(SomeInstance, SomeInstance)
+def intersection_Instance(s_inst1, s_inst2):
+    can_be_None = s_inst1.can_be_None and s_inst2.can_be_None
+    if s_inst1.classdef.issubclass(s_inst2.classdef):
+        return SomeInstance(s_inst1.classdef, can_be_None=can_be_None)
+    elif s_inst2.classdef.issubclass(s_inst1.classdef):
+        return SomeInstance(s_inst2.classdef, can_be_None=can_be_None)
+    else:
+        return s_ImpossibleValue
+
+@difference.register(SomeInstance, SomeInstance)
+def difference_Instance_Instance(s_inst1, s_inst2):
+    if s_inst1.classdef.issubclass(s_inst2.classdef):
+        return s_ImpossibleValue
+    else:
+        return s_inst1
+
+
+class SomeException(SomeObject):
+    """The set of exceptions obeying type(exc) in self.classes"""
+    def __init__(self, classdefs):
+        self.classdefs = classdefs
+
+    def as_SomeInstance(self):
+        return unionof(*[SomeInstance(cdef) for cdef in self.classdefs])
+
+@intersection.register(SomeException, SomeInstance)
+def intersection_Exception_Instance(s_exc, s_inst):
+    classdefs = {c for c in s_exc.classdefs if c.issubclass(s_inst.classdef)}
+    if classdefs:
+        return SomeException(classdefs)
+    else:
+        return s_ImpossibleValue
+
+@intersection.register(SomeInstance, SomeException)
+def intersection_Exception_Instance(s_inst, s_exc):
+    return intersection(s_exc, s_inst)
+
+@difference.register(SomeException, SomeInstance)
+def difference_Exception_Instance(s_exc, s_inst):
+    classdefs = {c for c in s_exc.classdefs
+        if not c.issubclass(s_inst.classdef)}
+    if classdefs:
+        return SomeException(classdefs)
+    else:
+        return s_ImpossibleValue
 
 class SomePBC(SomeObject):
     """Stands for a global user instance, built prior to the analysis,
@@ -682,14 +758,15 @@ def unionof(*somevalues):
 
 def add_knowntypedata(ktd, truth, vars, s_obj):
     for v in vars:
-        ktd[(truth, v)] = s_obj
+        ktd[truth][v] = s_obj
 
 
 def merge_knowntypedata(ktd1, ktd2):
-    r = {}
-    for truth_v in ktd1:
-        if truth_v in ktd2:
-            r[truth_v] = unionof(ktd1[truth_v], ktd2[truth_v])
+    r = defaultdict(dict)
+    for truth, constraints in ktd1.items():
+        for v in constraints:
+            if truth in ktd2 and v in ktd2[truth]:
+                r[truth][v] = unionof(ktd1[truth][v], ktd2[truth][v])
     return r
 
 
