@@ -285,7 +285,7 @@ class AbstractCPU(object):
 
 class CompiledLoopToken(object):
     asmmemmgr_blocks = None
-    asmmemmgr_gcroots = 0
+    asmmemmgr_gcreftracers = None
 
     def __init__(self, cpu, number):
         cpu.tracker.total_compiled_loops += 1
