@@ -8,7 +8,7 @@ from rpython.jit.backend.llsupport.descr import CallDescr, unpack_arraydescr
 from rpython.jit.backend.llsupport.gcmap import allocate_gcmap
 from rpython.jit.backend.llsupport.regalloc import (FrameManager, BaseRegalloc,
      RegisterManager, TempVar, compute_vars_longevity, is_comparison_or_ovf_op,
-     valid_addressing_size)
+     valid_addressing_size, get_scale)
 from rpython.jit.backend.x86 import rx86
 from rpython.jit.backend.x86.arch import (WORD, JITFRAME_FIXED_SIZE, IS_X86_32,
     IS_X86_64, DEFAULT_FRAME_BYTES)
@@ -31,6 +31,7 @@ from rpython.rlib.rarithmetic import r_longlong, r_uint
 from rpython.rtyper.annlowlevel import cast_instance_to_gcref
 from rpython.rtyper.lltypesystem import lltype, rffi, rstr
 from rpython.rtyper.lltypesystem.lloperation import llop
+from rpython.jit.backend.x86.regloc import AddressLoc
 
 
 class X86RegisterManager(RegisterManager):
@@ -357,11 +358,11 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
             assert self.assembler.mc._frame_size == DEFAULT_FRAME_BYTES
             self.rm.position = i
             self.xrm.position = i
-            if op.has_no_side_effect() and op not in self.longevity:
+            if rop.has_no_side_effect(op.opnum) and op not in self.longevity:
                 i += 1
                 self.possibly_free_vars_for_op(op)
                 continue
-            if not we_are_translated() and op.getopnum() == -127:
+            if not we_are_translated() and op.getopnum() == rop.FORCE_SPILL:
                 self._consider_force_spill(op)
             else:
                 oplist[op.getopnum()](self, op)
@@ -422,16 +423,11 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
     def consider_finish(self, op):
         # the frame is in ebp, but we have to point where in the frame is
         # the potential argument to FINISH
-        descr = op.getdescr()
-        fail_descr = cast_instance_to_gcref(descr)
-        # we know it does not move, but well
-        rgc._make_sure_does_not_move(fail_descr)
-        fail_descr = rffi.cast(lltype.Signed, fail_descr)
         if op.numargs() == 1:
             loc = self.make_sure_var_in_reg(op.getarg(0))
-            locs = [loc, imm(fail_descr)]
+            locs = [loc]
         else:
-            locs = [imm(fail_descr)]
+            locs = []
         self.perform(op, locs, None)
 
     def consider_guard_no_exception(self, op):
@@ -1008,7 +1004,7 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
         self.rm.possibly_free_var(length_box)
         #
         itemsize = op.getarg(1).getint()
-        maxlength = (gc_ll_descr.max_size_of_young_obj - WORD * 2) / itemsize
+        maxlength = (gc_ll_descr.max_size_of_young_obj - WORD * 2)
         self.assembler.malloc_cond_varsize(
             op.getarg(0).getint(),
             gc_ll_descr.get_nursery_free_addr(),
@@ -1140,6 +1136,10 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
     consider_same_as_r = _consider_same_as
     consider_same_as_f = _consider_same_as
 
+    def consider_load_from_gc_table(self, op):
+        resloc = self.rm.force_allocate_reg(op)
+        self.perform(op, [], resloc)
+
     def consider_int_force_ge_zero(self, op):
         argloc = self.make_sure_var_in_reg(op.getarg(0))
         resloc = self.force_allocate_reg(op, [op.getarg(0)])
@@ -1388,21 +1388,39 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
     def consider_keepalive(self, op):
         pass
 
-    def consider_zero_array(self, op):
-        itemsize, baseofs, _ = unpack_arraydescr(op.getdescr())
-        length_box = op.getarg(2)
-        if isinstance(length_box, ConstInt):
-            constbytes = length_box.getint() * itemsize
-            if constbytes == 0:
-                return    # nothing to do
+    def _scaled_addr(self, index_loc, itemsize_loc,
+                                base_loc, ofs_loc):
+        assert isinstance(itemsize_loc, ImmedLoc)
+        itemsize = itemsize_loc.value
+        if isinstance(index_loc, ImmedLoc):
+            temp_loc = imm(index_loc.value * itemsize)
+            shift = 0
         else:
-            constbytes = -1
+            assert valid_addressing_size(itemsize), "rewrite did not correctly handle shift/mul!"
+            temp_loc = index_loc
+            shift = get_scale(itemsize)
+        assert isinstance(ofs_loc, ImmedLoc)
+        return AddressLoc(base_loc, temp_loc, shift, ofs_loc.value)
+
+    def consider_zero_array(self, op):
+        _, baseofs, _ = unpack_arraydescr(op.getdescr())
+        length_box = op.getarg(2)
+
+        scale_box = op.getarg(3)
+        assert isinstance(scale_box, ConstInt)
+        start_itemsize = scale_box.value
+
+        len_scale_box = op.getarg(4)
+        assert isinstance(len_scale_box, ConstInt)
+        len_itemsize = len_scale_box.value
+        # rewrite handles the mul of a constant length box
+        constbytes = -1
+        if isinstance(length_box, ConstInt):
+            constbytes = length_box.getint()
         args = op.getarglist()
         base_loc = self.rm.make_sure_var_in_reg(args[0], args)
         startindex_loc = self.rm.make_sure_var_in_reg(args[1], args)
-        if 0 <= constbytes <= 16 * 8 and (
-                valid_addressing_size(itemsize) or
-                isinstance(startindex_loc, ImmedLoc)):
+        if 0 <= constbytes <= 16 * 8:
             if IS_X86_64:
                 null_loc = X86_64_XMM_SCRATCH_REG
             else:
@@ -1410,7 +1428,7 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
                 null_loc = self.xrm.force_allocate_reg(null_box)
                 self.xrm.possibly_free_var(null_box)
             self.perform_discard(op, [base_loc, startindex_loc,
-                                      imm(constbytes), imm(itemsize),
+                                      imm(constbytes), imm(start_itemsize),
                                       imm(baseofs), null_loc])
         else:
             # base_loc and startindex_loc are in two regs here (or they are
@@ -1420,10 +1438,9 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
             # args[2], because we're still needing the latter.
             dstaddr_box = TempVar()
             dstaddr_loc = self.rm.force_allocate_reg(dstaddr_box, [args[2]])
-            itemsize_loc = imm(itemsize)
-            dst_addr = self.assembler._get_interiorfield_addr(
-                dstaddr_loc, startindex_loc, itemsize_loc,
-                base_loc, imm(baseofs))
+            itemsize_loc = imm(start_itemsize)
+            dst_addr = self._scaled_addr(startindex_loc, itemsize_loc,
+                                         base_loc, imm(baseofs))
             self.assembler.mc.LEA(dstaddr_loc, dst_addr)
             #
             if constbytes >= 0:
@@ -1432,15 +1449,15 @@ class RegAlloc(BaseRegalloc, VectorRegallocMixin):
                 # load length_loc in a register different than dstaddr_loc
                 length_loc = self.rm.make_sure_var_in_reg(length_box,
                                                           [dstaddr_box])
-                if itemsize > 1:
+                if len_itemsize > 1:
                     # we need a register that is different from dstaddr_loc,
                     # but which can be identical to length_loc (as usual,
                     # only if the length_box is not used by future operations)
                     bytes_box = TempVar()
                     bytes_loc = self.rm.force_allocate_reg(bytes_box,
                                                            [dstaddr_box])
-                    b_adr = self.assembler._get_interiorfield_addr(
-                        bytes_loc, length_loc, itemsize_loc, imm0, imm0)
+                    len_itemsize_loc = imm(len_itemsize)
+                    b_adr = self._scaled_addr(length_loc, len_itemsize_loc, imm0, imm0)
                     self.assembler.mc.LEA(bytes_loc, b_adr)
                     length_box = bytes_box
                     length_loc = bytes_loc
