@@ -1,33 +1,59 @@
-from rpython.jit.metainterp.history import ConstInt
+from rpython.jit.metainterp.history import Const, ConstInt
+from rpython.jit.metainterp.history import FrontendOp, RefFrontendOp
 from rpython.jit.metainterp.resoperation import rop, OpHelpers
+from rpython.jit.metainterp.executor import constant_from_op
+from rpython.rlib.rarithmetic import r_uint32, r_uint
+from rpython.rlib.objectmodel import always_inline
 
-class HeapCacheValue(object):
-    def __init__(self, box):
-        self.box = box
-        self.likely_virtual = False
-        self.reset_keep_likely_virtual()
+""" A big note: we don't do heap caches on Consts, because it used
+to be done with the identity of the Const instance. This gives very wonky
+results at best, so we decided to not do it at all. Can be fixed with
+interning of Consts (already done on trace anyway)
+"""
 
-    def reset_keep_likely_virtual(self):
-        self.known_class = False
-        self.known_nullity = False
-        # did we see the allocation during tracing?
-        self.seen_allocation = False
-        self.is_unescaped = False
-        self.nonstandard_virtualizable = False
-        self.length = None
-        self.dependencies = None
+# RefFrontendOp._heapc_flags:
+HF_LIKELY_VIRTUAL  = 0x01
+HF_KNOWN_CLASS     = 0x02
+HF_KNOWN_NULLITY   = 0x04
+HF_SEEN_ALLOCATION = 0x08   # did we see the allocation during tracing?
+HF_IS_UNESCAPED    = 0x10
+HF_NONSTD_VABLE    = 0x20
 
-    def __repr__(self):
-        return 'HeapCacheValue(%s)' % (self.box, )
+_HF_VERSION_INC    = 0x40   # must be last
+_HF_VERSION_MAX    = r_uint(2 ** 32 - _HF_VERSION_INC)
+
+@always_inline
+def add_flags(ref_frontend_op, flags):
+    f = ref_frontend_op._get_heapc_flags()
+    f |= r_uint(flags)
+    ref_frontend_op._set_heapc_flags(f)
+
+@always_inline
+def remove_flags(ref_frontend_op, flags):
+    f = ref_frontend_op._get_heapc_flags()
+    f &= r_uint(~flags)
+    ref_frontend_op._set_heapc_flags(f)
+
+@always_inline
+def test_flags(ref_frontend_op, flags):
+    f = ref_frontend_op._get_heapc_flags()
+    return bool(f & r_uint(flags))
+
+def maybe_replace_with_const(box):
+    if not isinstance(box, Const) and box.is_replaced_with_const():
+        return constant_from_op(box)
+    else:
+        return box
 
 
 class CacheEntry(object):
-    def __init__(self):
-        # both are {from_value: to_value} dicts
+    def __init__(self, heapcache):
+        # both are {from_ref_box: to_field_box} dicts
         # the first is for boxes where we did not see the allocation, the
         # second for anything else. the reason that distinction makes sense is
         # because if we saw the allocation, we know it cannot alias with
         # anything else where we saw the allocation.
+        self.heapcache = heapcache
         self.cache_anything = {}
         self.cache_seen_allocation = {}
 
@@ -36,112 +62,137 @@ class CacheEntry(object):
             self.cache_seen_allocation.clear()
         self.cache_anything.clear()
 
-    def _getdict(self, value):
-        if value.seen_allocation:
+    def _seen_alloc(self, ref_box):
+        if not isinstance(ref_box, RefFrontendOp):
+            return False
+        return self.heapcache._check_flag(ref_box, HF_SEEN_ALLOCATION)
+
+    def _getdict(self, seen_alloc):
+        if seen_alloc:
             return self.cache_seen_allocation
         else:
             return self.cache_anything
 
-    def do_write_with_aliasing(self, value, fieldvalue):
-        self._clear_cache_on_write(value.seen_allocation)
-        self._getdict(value)[value] = fieldvalue
+    def do_write_with_aliasing(self, ref_box, fieldbox):
+        seen_alloc = self._seen_alloc(ref_box)
+        self._clear_cache_on_write(seen_alloc)
+        self._getdict(seen_alloc)[ref_box] = fieldbox
 
-    def read(self, value):
-        return self._getdict(value).get(value, None)
+    def read(self, ref_box):
+        dict = self._getdict(self._seen_alloc(ref_box))
+        try:
+            res_box = dict[ref_box]
+        except KeyError:
+            return None
+        return maybe_replace_with_const(res_box)
 
-    def read_now_known(self, value, fieldvalue):
-        self._getdict(value)[value] = fieldvalue
+    def read_now_known(self, ref_box, fieldbox):
+        self._getdict(self._seen_alloc(ref_box))[ref_box] = fieldbox
 
     def invalidate_unescaped(self):
         self._invalidate_unescaped(self.cache_anything)
         self._invalidate_unescaped(self.cache_seen_allocation)
 
     def _invalidate_unescaped(self, d):
-        for value in d.keys():
-            if not value.is_unescaped:
-                del d[value]
+        for ref_box in d.keys():
+            if not self.heapcache.is_unescaped(ref_box):
+                del d[ref_box]
 
 
 class FieldUpdater(object):
-    def __init__(self, heapcache, value, cache, fieldvalue):
-        self.heapcache = heapcache
-        self.value = value
+    def __init__(self, ref_box, cache, fieldbox):
+        self.ref_box = ref_box
         self.cache = cache
-        if fieldvalue is not None:
-            self.currfieldbox = fieldvalue.box
-        else:
-            self.currfieldbox = None
+        self.currfieldbox = fieldbox     # <= read directly from pyjitpl.py
 
     def getfield_now_known(self, fieldbox):
-        fieldvalue = self.heapcache.getvalue(fieldbox)
-        self.cache.read_now_known(self.value, fieldvalue)
+        self.cache.read_now_known(self.ref_box, fieldbox)
 
     def setfield(self, fieldbox):
-        fieldvalue = self.heapcache.getvalue(fieldbox)
-        self.cache.do_write_with_aliasing(self.value, fieldvalue)
+        self.cache.do_write_with_aliasing(self.ref_box, fieldbox)
+
+class DummyFieldUpdater(FieldUpdater):
+    def __init__(self):
+        self.currfieldbox = None
+
+    def getfield_now_known(self, fieldbox):
+        pass
+
+    def setfield(self, fieldbox):
+        pass
+
+dummy_field_updater = DummyFieldUpdater()
 
 
 class HeapCache(object):
     def __init__(self):
+        # Works with flags stored on RefFrontendOp._heapc_flags.
+        # There are two ways to do a global resetting of these flags:
+        # reset() and reset_keep_likely_virtual().  The basic idea is
+        # to use a version number in each RefFrontendOp, and in order
+        # to reset the flags globally, we increment the global version
+        # number in this class.  Then when we read '_heapc_flags' we
+        # also check if the associated version number is up-to-date
+        # or not.  More precisely, we have two global version numbers
+        # here: 'head_version' and 'likely_virtual_version'.  Normally
+        # we use 'head_version'.  For is_likely_virtual() though, we
+        # use the other, older version number.
+        self.head_version = r_uint(0)
+        self.likely_virtual_version = r_uint(0)
         self.reset()
 
     def reset(self):
-        # maps boxes to values
-        self.values = {}
-        # store the boxes that contain newly allocated objects, this maps the
-        # boxes to a bool, the bool indicates whether or not the object has
-        # escaped the trace or not (True means the box never escaped, False
-        # means it did escape), its presences in the mapping shows that it was
-        # allocated inside the trace
-        #if trace_branch:
-            #self.new_boxes = {}
-        #    pass
-        #else:
-            #for box in self.new_boxes:
-            #    self.new_boxes[box] = False
-        #    pass
-        #if reset_virtuals:
-        #    self.likely_virtuals = {}      # only for jit.isvirtual()
-        # Tracks which boxes should be marked as escaped when the key box
-        # escapes.
-        #self.dependencies = {}
-
+        # Global reset of all flags.  Update both version numbers so
+        # that any access to '_heapc_flags' will be marked as outdated.
+        assert self.head_version < _HF_VERSION_MAX
+        self.head_version += _HF_VERSION_INC
+        self.likely_virtual_version = self.head_version
+        #
         # heap cache
         # maps descrs to CacheEntry
         self.heap_cache = {}
         # heap array cache
-        # maps descrs to {index: {from_value: to_value}} dicts
+        # maps descrs to {index: CacheEntry} dicts
         self.heap_array_cache = {}
 
     def reset_keep_likely_virtuals(self):
-        for value in self.values.itervalues():
-            value.reset_keep_likely_virtual()
+        # Update only 'head_version', but 'likely_virtual_version' remains
+        # at its older value.
+        assert self.head_version < _HF_VERSION_MAX
+        self.head_version += _HF_VERSION_INC
         self.heap_cache = {}
         self.heap_array_cache = {}
 
-    def getvalue(self, box, create=True):
-        value = self.values.get(box, None)
-        if not value and create:
-            value = self.values[box] = HeapCacheValue(box)
-        return value
+    @always_inline
+    def test_head_version(self, ref_frontend_op):
+        return ref_frontend_op._get_heapc_flags() >= self.head_version
 
-    def getvalues(self, boxes):
-        return [self.getvalue(box) for box in boxes]
+    @always_inline
+    def test_likely_virtual_version(self, ref_frontend_op):
+        return ref_frontend_op._get_heapc_flags() >= self.likely_virtual_version
+
+    def update_version(self, ref_frontend_op):
+        """Ensure the version of 'ref_frontend_op' is current.  If not,
+        it will update 'ref_frontend_op' (removing most flags currently set).
+        """
+        if not self.test_head_version(ref_frontend_op):
+            f = self.head_version
+            if (self.test_likely_virtual_version(ref_frontend_op) and
+                test_flags(ref_frontend_op, HF_LIKELY_VIRTUAL)):
+                f |= HF_LIKELY_VIRTUAL
+            ref_frontend_op._set_heapc_flags(f)
+            ref_frontend_op._heapc_deps = None
 
     def invalidate_caches(self, opnum, descr, argboxes):
         self.mark_escaped(opnum, descr, argboxes)
         self.clear_caches(opnum, descr, argboxes)
 
     def _escape_from_write(self, box, fieldbox):
-        value = self.getvalue(box, create=False)
-        fieldvalue = self.getvalue(fieldbox, create=False)
-        if (value is not None and value.is_unescaped and
-                fieldvalue is not None and fieldvalue.is_unescaped):
-            if value.dependencies is None:
-                value.dependencies = []
-            value.dependencies.append(fieldvalue)
-        elif fieldvalue is not None:
-            self._escape(fieldvalue)
+        if self.is_unescaped(box) and self.is_unescaped(fieldbox):
+            deps = self._get_deps(box)
+            deps.append(fieldbox)
+        elif fieldbox is not None:
+            self._escape_box(fieldbox)
 
     def mark_escaped(self, opnum, descr, argboxes):
         if opnum == rop.SETFIELD_GC:
@@ -176,19 +227,20 @@ class HeapCache(object):
                 self._escape_box(box)
 
     def _escape_box(self, box):
-        value = self.getvalue(box, create=False)
-        if not value:
-            return
-        self._escape(value)
-
-    def _escape(self, value):
-        value.is_unescaped = False
-        value.likely_virtual = False
-        deps = value.dependencies
-        value.dependencies = None
-        if deps is not None:
-            for dep in deps:
-                self._escape(dep)
+        if isinstance(box, RefFrontendOp):
+            remove_flags(box, HF_LIKELY_VIRTUAL | HF_IS_UNESCAPED)
+            deps = box._heapc_deps
+            if deps is not None:
+                if not self.test_head_version(box):
+                    box._heapc_deps = None
+                else:
+                    # 'deps[0]' is abused to store the array length, keep it
+                    if deps[0] is None:
+                        box._heapc_deps = None
+                    else:
+                        box._heapc_deps = [deps[0]]
+                    for i in range(1, len(deps)):
+                        self._escape_box(deps[i])
 
     def clear_caches(self, opnum, descr, argboxes):
         if (opnum == rop.SETFIELD_GC or
@@ -241,7 +293,8 @@ class HeapCache(object):
         self.reset_keep_likely_virtuals()
 
     def _clear_caches_arraycopy(self, opnum, desrc, argboxes, effectinfo):
-        seen_allocation_of_target = self.getvalue(argboxes[2]).seen_allocation
+        seen_allocation_of_target = self._check_flag(
+                                            argboxes[2], HF_SEEN_ALLOCATION)
         if (
             isinstance(argboxes[3], ConstInt) and
             isinstance(argboxes[4], ConstInt) and
@@ -285,74 +338,82 @@ class HeapCache(object):
             return
         self.reset_keep_likely_virtuals()
 
+    def _get_deps(self, box):
+        if not isinstance(box, RefFrontendOp):
+            return None
+        self.update_version(box)
+        if box._heapc_deps is None:
+            box._heapc_deps = [None]
+        return box._heapc_deps
+
+    def _check_flag(self, box, flag):
+        return (isinstance(box, RefFrontendOp) and
+                    self.test_head_version(box) and
+                    test_flags(box, flag))
+
+    def _set_flag(self, box, flag):
+        assert isinstance(box, RefFrontendOp)
+        self.update_version(box)
+        add_flags(box, flag)
+
     def is_class_known(self, box):
-        value = self.getvalue(box, create=False)
-        if value:
-            return value.known_class
-        return False
+        return self._check_flag(box, HF_KNOWN_CLASS)
 
     def class_now_known(self, box):
-        self.getvalue(box).known_class = True
+        if isinstance(box, Const):
+            return
+        self._set_flag(box, HF_KNOWN_CLASS)
 
     def is_nullity_known(self, box):
-        value = self.getvalue(box, create=False)
-        if value:
-            return value.known_nullity
-        return False
+        if isinstance(box, Const):
+            return bool(box.getref_base())
+        return self._check_flag(box, HF_KNOWN_NULLITY)
 
     def nullity_now_known(self, box):
-        self.getvalue(box).known_nullity = True
+        if isinstance(box, Const):
+            return
+        self._set_flag(box, HF_KNOWN_NULLITY)
 
     def is_nonstandard_virtualizable(self, box):
-        value = self.getvalue(box, create=False)
-        if value:
-            return value.nonstandard_virtualizable
-        return False
+        return self._check_flag(box, HF_NONSTD_VABLE)
 
     def nonstandard_virtualizables_now_known(self, box):
-        self.getvalue(box).nonstandard_virtualizable = True
+        self._set_flag(box, HF_NONSTD_VABLE)
 
     def is_unescaped(self, box):
-        value = self.getvalue(box, create=False)
-        if value:
-            return value.is_unescaped
-        return False
+        return self._check_flag(box, HF_IS_UNESCAPED)
 
     def is_likely_virtual(self, box):
-        value = self.getvalue(box, create=False)
-        if value:
-            return value.likely_virtual
-        return False
+        # note: this is different from _check_flag()
+        return (isinstance(box, RefFrontendOp) and
+                self.test_likely_virtual_version(box) and
+                test_flags(box, HF_LIKELY_VIRTUAL))
 
     def new(self, box):
-        value = self.getvalue(box)
-        value.is_unescaped = True
-        value.likely_virtual = True
-        value.seen_allocation = True
+        assert isinstance(box, RefFrontendOp)
+        self.update_version(box)
+        add_flags(box, HF_LIKELY_VIRTUAL | HF_SEEN_ALLOCATION | HF_IS_UNESCAPED)
 
     def new_array(self, box, lengthbox):
         self.new(box)
         self.arraylen_now_known(box, lengthbox)
 
     def getfield(self, box, descr):
-        value = self.getvalue(box, create=False)
-        if value:
-            cache = self.heap_cache.get(descr, None)
-            if cache:
-                tovalue = cache.read(value)
-                if tovalue:
-                    return tovalue.box
+        cache = self.heap_cache.get(descr, None)
+        if cache:
+            return cache.read(box)
         return None
 
     def get_field_updater(self, box, descr):
-        value = self.getvalue(box)
+        if not isinstance(box, RefFrontendOp):
+            return dummy_field_updater
         cache = self.heap_cache.get(descr, None)
         if cache is None:
-            cache = self.heap_cache[descr] = CacheEntry()
-            fieldvalue = None
+            cache = self.heap_cache[descr] = CacheEntry(self)
+            fieldbox = None
         else:
-            fieldvalue = cache.read(value)
-        return FieldUpdater(self, value, cache, fieldvalue)
+            fieldbox = cache.read(box)
+        return FieldUpdater(box, cache, fieldbox)
 
     def getfield_now_known(self, box, descr, fieldbox):
         upd = self.get_field_updater(box, descr)
@@ -365,17 +426,12 @@ class HeapCache(object):
     def getarrayitem(self, box, indexbox, descr):
         if not isinstance(indexbox, ConstInt):
             return None
-        value = self.getvalue(box, create=False)
-        if value is None:
-            return None
         index = indexbox.getint()
         cache = self.heap_array_cache.get(descr, None)
         if cache:
             indexcache = cache.get(index, None)
             if indexcache is not None:
-                resvalue = indexcache.read(value)
-                if resvalue:
-                    return resvalue.box
+                return indexcache.read(box)
         return None
 
     def _get_or_make_array_cache_entry(self, indexbox, descr):
@@ -385,16 +441,14 @@ class HeapCache(object):
         cache = self.heap_array_cache.setdefault(descr, {})
         indexcache = cache.get(index, None)
         if indexcache is None:
-            cache[index] = indexcache = CacheEntry()
+            cache[index] = indexcache = CacheEntry(self)
         return indexcache
 
 
     def getarrayitem_now_known(self, box, indexbox, fieldbox, descr):
-        value = self.getvalue(box)
-        fieldvalue = self.getvalue(fieldbox)
         indexcache = self._get_or_make_array_cache_entry(indexbox, descr)
         if indexcache:
-            indexcache.read_now_known(value, fieldvalue)
+            indexcache.read_now_known(box, fieldbox)
 
     def setarrayitem(self, box, indexbox, fieldbox, descr):
         if not isinstance(indexbox, ConstInt):
@@ -402,25 +456,31 @@ class HeapCache(object):
             if cache is not None:
                 cache.clear()
             return
-        value = self.getvalue(box)
-        fieldvalue = self.getvalue(fieldbox)
         indexcache = self._get_or_make_array_cache_entry(indexbox, descr)
         if indexcache:
-            indexcache.do_write_with_aliasing(value, fieldvalue)
+            indexcache.do_write_with_aliasing(box, fieldbox)
 
     def arraylen(self, box):
-        value = self.getvalue(box, create=False)
-        if value and value.length:
-            return value.length.box
+        if (isinstance(box, RefFrontendOp) and
+            self.test_head_version(box) and
+            box._heapc_deps is not None):
+            res_box = box._heapc_deps[0]
+            if res_box is not None:
+                return maybe_replace_with_const(res_box)
         return None
 
     def arraylen_now_known(self, box, lengthbox):
-        value = self.getvalue(box)
-        value.length = self.getvalue(lengthbox)
+        # we store in '_heapc_deps' a list of boxes: the *first* box is
+        # the known length or None, and the remaining boxes are the
+        # regular dependencies.
+        if isinstance(box, Const):
+            return
+        deps = self._get_deps(box)
+        assert deps is not None
+        deps[0] = lengthbox
 
     def replace_box(self, oldbox, newbox):
-        value = self.getvalue(oldbox, create=False)
-        if value is None:
-            return
-        value.box = newbox
-        self.values[newbox] = value
+        # here, only for replacing a box with a const
+        if isinstance(oldbox, FrontendOp) and isinstance(newbox, Const):
+            assert newbox.same_constant(constant_from_op(oldbox))
+            oldbox.set_replaced_with_const()
