@@ -1,15 +1,9 @@
+import py
 from rpython.jit.metainterp.heapcache import HeapCache
-from rpython.jit.metainterp.resoperation import rop
-from rpython.jit.metainterp.history import ConstInt, BoxInt, BasicFailDescr
+from rpython.jit.metainterp.resoperation import rop, InputArgInt
+from rpython.jit.metainterp.history import ConstInt, ConstPtr, BasicFailDescr
+from rpython.jit.metainterp.history import IntFrontendOp, RefFrontendOp
 
-box1 = "box1"
-box2 = "box2"
-box3 = "box3"
-box4 = "box4"
-box5 = "box5"
-lengthbox1 = object()
-lengthbox2 = object()
-lengthbox3 = object()
 descr1 = object()
 descr2 = object()
 descr3 = object()
@@ -59,43 +53,52 @@ arraycopydescr1 = FakeCallDescr(FakeEffectinfo.EF_CANNOT_RAISE, FakeEffectinfo.O
 class TestHeapCache(object):
     def test_known_class_box(self):
         h = HeapCache()
-        assert not h.is_class_known(1)
-        assert not h.is_class_known(2)
-        h.class_now_known(1)
-        assert h.is_class_known(1)
-        assert not h.is_class_known(2)
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        assert not h.is_class_known(box1)
+        assert not h.is_class_known(box2)
+        h.class_now_known(box1)
+        assert h.is_class_known(box1)
+        assert not h.is_class_known(box2)
 
         h.reset()
-        assert not h.is_class_known(1)
-        assert not h.is_class_known(2)
+        assert not h.is_class_known(box1)
+        assert not h.is_class_known(box2)
 
     def test_known_nullity(self):
         h = HeapCache()
-        assert not h.is_nullity_known(1)
-        assert not h.is_nullity_known(2)
-        h.nullity_now_known(1)
-        assert h.is_nullity_known(1)
-        assert not h.is_nullity_known(2)
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        assert not h.is_nullity_known(box1)
+        assert not h.is_nullity_known(box2)
+        h.nullity_now_known(box1)
+        assert h.is_nullity_known(box1)
+        assert not h.is_nullity_known(box2)
 
         h.reset()
-        assert not h.is_nullity_known(1)
-        assert not h.is_nullity_known(2)
+        assert not h.is_nullity_known(box1)
+        assert not h.is_nullity_known(box2)
 
 
     def test_nonstandard_virtualizable(self):
         h = HeapCache()
-        assert not h.is_nonstandard_virtualizable(1)
-        assert not h.is_nonstandard_virtualizable(2)
-        h.nonstandard_virtualizables_now_known(1)
-        assert h.is_nonstandard_virtualizable(1)
-        assert not h.is_nonstandard_virtualizable(2)
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        assert not h.is_nonstandard_virtualizable(box1)
+        assert not h.is_nonstandard_virtualizable(box2)
+        h.nonstandard_virtualizables_now_known(box1)
+        assert h.is_nonstandard_virtualizable(box1)
+        assert not h.is_nonstandard_virtualizable(box2)
 
         h.reset()
-        assert not h.is_nonstandard_virtualizable(1)
-        assert not h.is_nonstandard_virtualizable(2)
+        assert not h.is_nonstandard_virtualizable(box1)
+        assert not h.is_nonstandard_virtualizable(box2)
 
     def test_heapcache_fields(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
         assert h.getfield(box1, descr1) is None
         assert h.getfield(box1, descr2) is None
         h.setfield(box1, box2, descr1)
@@ -119,6 +122,10 @@ class TestHeapCache(object):
 
     def test_heapcache_read_fields_multiple(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.getfield_now_known(box1, descr1, box2)
         h.getfield_now_known(box3, descr1, box4)
         assert h.getfield(box1, descr1) is box2
@@ -134,6 +141,10 @@ class TestHeapCache(object):
 
     def test_heapcache_write_fields_multiple(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
         h.setfield(box3, box4, descr1)
@@ -141,6 +152,10 @@ class TestHeapCache(object):
         assert h.getfield(box1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.new(box1)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
@@ -149,6 +164,10 @@ class TestHeapCache(object):
         assert h.getfield(box1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.new(box1)
         h.new(box3)
         h.setfield(box1, box2, descr1)
@@ -162,6 +181,10 @@ class TestHeapCache(object):
 
     def test_heapcache_arrays(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         assert h.getarrayitem(box1, index1, descr1) is None
         assert h.getarrayitem(box1, index1, descr2) is None
         assert h.getarrayitem(box1, index2, descr1) is None
@@ -204,6 +227,10 @@ class TestHeapCache(object):
 
     def test_heapcache_array_nonconst_index(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index2, box4, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -214,6 +241,10 @@ class TestHeapCache(object):
 
     def test_heapcache_read_fields_multiple_array(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.getarrayitem_now_known(box1, index1, box2, descr1)
         h.getarrayitem_now_known(box3, index1, box4, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -229,6 +260,10 @@ class TestHeapCache(object):
 
     def test_heapcache_write_fields_multiple_array(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.setarrayitem(box1, index1, box2, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
         h.setarrayitem(box3, index1, box4, descr1)
@@ -236,6 +271,10 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.new(box1)
         h.setarrayitem(box1, index1, box2, descr1)
         assert h.getarrayitem(box1, index1, descr1) is box2
@@ -244,6 +283,10 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index1, descr1) is None # box1 and box3 can alias
 
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.new(box1)
         h.new(box3)
         h.setarrayitem(box1, index1, box2, descr1)
@@ -257,6 +300,10 @@ class TestHeapCache(object):
 
     def test_length_cache(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        lengthbox1 = IntFrontendOp(11)
+        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         assert h.arraylen(box1) is lengthbox1
 
@@ -267,6 +314,9 @@ class TestHeapCache(object):
 
     def test_invalidate_cache(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index2, box4, descr1)
@@ -279,7 +329,7 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index2, descr1) is box4
 
         h.invalidate_caches(
-            rop.CALL, FakeCallDescr(FakeEffectinfo.EF_ELIDABLE_CANNOT_RAISE), [])
+            rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_ELIDABLE_CANNOT_RAISE), [])
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
@@ -290,16 +340,21 @@ class TestHeapCache(object):
         assert h.getarrayitem(box1, index2, descr1) is box4
 
         h.invalidate_caches(
-            rop.CALL_LOOPINVARIANT, FakeCallDescr(FakeEffectinfo.EF_LOOPINVARIANT), [])
+            rop.CALL_LOOPINVARIANT_N, FakeCallDescr(FakeEffectinfo.EF_LOOPINVARIANT), [])
 
         h.invalidate_caches(
-            rop.CALL, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
+            rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert h.getfield(box1, descr1) is None
         assert h.getarrayitem(box1, index1, descr1) is None
         assert h.getarrayitem(box1, index2, descr1) is None
 
-    def test_replace_box(self):
+    def test_replace_box_with_box(self):
+        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
         h.setfield(box2, box3, descr3)
@@ -310,16 +365,22 @@ class TestHeapCache(object):
         h.setfield(box4, box3, descr1)
         assert h.getfield(box4, descr1) is box3
 
+    def test_replace_box_with_const(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        c_box3 = ConstPtr(ConstPtr.value)
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
         h.setfield(box2, box3, descr3)
-        h.replace_box(box3, box4)
+        h.replace_box(box3, c_box3)
         assert h.getfield(box1, descr1) is box2
-        assert h.getfield(box1, descr2) is box4
-        assert h.getfield(box2, descr3) is box4
+        assert c_box3.same_constant(h.getfield(box1, descr2))
+        assert c_box3.same_constant(h.getfield(box2, descr3))
 
     def test_replace_box_twice(self):
+        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setfield(box1, box2, descr1)
         h.setfield(box1, box3, descr2)
@@ -343,6 +404,7 @@ class TestHeapCache(object):
         assert h.getfield(box2, descr3) is box5
 
     def test_replace_box_array(self):
+        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index1, box3, descr2)
@@ -362,6 +424,7 @@ class TestHeapCache(object):
         assert h.arraylen(box4) is lengthbox2
 
     def test_replace_box_array_twice(self):
+        py.test.skip("replacing a box with another box: not supported any more")
         h = HeapCache()
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index1, box3, descr2)
@@ -382,20 +445,46 @@ class TestHeapCache(object):
         h.replace_box(lengthbox2, lengthbox3)
         assert h.arraylen(box4) is lengthbox3
 
+    def test_replace_box_with_const_in_array(self):
+        h = HeapCache()
+        box1 = RefFrontendOp(1)
+        lengthbox2 = IntFrontendOp(2)
+        lengthbox2.setint(10)
+        h.arraylen_now_known(box1, lengthbox2)
+        assert h.arraylen(box1) is lengthbox2
+        c10 = ConstInt(10)
+        h.replace_box(lengthbox2, c10)
+        assert c10.same_constant(h.arraylen(box1))
+
+        box2 = IntFrontendOp(2)
+        box2.setint(12)
+        h.setarrayitem(box1, index2, box2, descr1)
+        assert h.getarrayitem(box1, index2, descr1) is box2
+        c12 = ConstInt(12)
+        h.replace_box(box2, c12)
+        assert c12.same_constant(h.getarrayitem(box1, index2, descr1))
+
     def test_ll_arraycopy(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
+        box5 = RefFrontendOp(5)
+        lengthbox1 = IntFrontendOp(11)
+        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         h.setarrayitem(box1, index1, box2, descr1)
         h.new_array(box2, lengthbox1)
         # Just need the destination box for this call
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box5, box2, index1, index1, index1]
         )
         assert h.getarrayitem(box1, index1, descr1) is box2
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box5, box3, index1, index1, index1]
         )
@@ -404,7 +493,7 @@ class TestHeapCache(object):
         h.setarrayitem(box4, index1, box2, descr1)
         assert h.getarrayitem(box4, index1, descr1) is box2
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box3, box5, index1, index1, index2]
         )
@@ -412,11 +501,15 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_differing_descrs(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        lengthbox2 = IntFrontendOp(12)
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
         h.new_array(box2, lengthbox2)
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box3, box2, index1, index1, index2]
         )
@@ -424,20 +517,26 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_differing_descrs_nonconst_index(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
-            [None, box3, box2, index1, index1, BoxInt()]
+            [None, box3, box2, index1, index1, InputArgInt()]
         )
         assert h.getarrayitem(box1, index1, descr2) is box2
 
     def test_ll_arraycopy_result_propogated(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr1)
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box1, box3, index1, index1, index2]
         )
@@ -445,35 +544,46 @@ class TestHeapCache(object):
 
     def test_ll_arraycopy_dest_new(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
+        lengthbox1 = IntFrontendOp(11)
         h.new_array(box1, lengthbox1)
         h.setarrayitem(box3, index1, box4, descr1)
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
         )
 
     def test_ll_arraycopy_doesnt_escape_arrays(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        lengthbox1 = IntFrontendOp(11)
+        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         h.new_array(box2, lengthbox2)
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
         )
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         h.invalidate_caches(
-            rop.CALL,
+            rop.CALL_N,
             arraycopydescr1,
-            [None, box2, box1, index1, index1, BoxInt()]
+            [None, box2, box1, index1, index1, InputArgInt()]
         )
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
 
     def test_unescaped(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         assert not h.is_unescaped(box1)
         h.new(box2)
         assert h.is_unescaped(box2)
@@ -484,6 +594,9 @@ class TestHeapCache(object):
 
     def test_unescaped_testing(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
         h.new(box1)
         h.new(box2)
         assert h.is_unescaped(box1)
@@ -492,7 +605,7 @@ class TestHeapCache(object):
         h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert h.is_unescaped(box2)
         # Reading a field from a virtual doesn't escape it.
-        h.invalidate_caches(rop.GETFIELD_GC, None, [box1])
+        h.invalidate_caches(rop.GETFIELD_GC_I, None, [box1])
         assert h.is_unescaped(box1)
         # Escaping a virtual transitively escapes anything inside of it.
         assert not h.is_unescaped(box3)
@@ -502,6 +615,8 @@ class TestHeapCache(object):
 
     def test_ops_dont_escape(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         assert h.is_unescaped(box1)
@@ -515,6 +630,9 @@ class TestHeapCache(object):
 
     def test_circular_virtuals(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        box3 = RefFrontendOp(3)
         h.new(box1)
         h.new(box2)
         h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
@@ -523,6 +641,10 @@ class TestHeapCache(object):
 
     def test_unescaped_array(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
+        lengthbox1 = IntFrontendOp(11)
+        lengthbox2 = IntFrontendOp(12)
         h.new_array(box1, lengthbox1)
         assert h.is_unescaped(box1)
         h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box1, index1, box2])
@@ -539,17 +661,19 @@ class TestHeapCache(object):
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         h.invalidate_caches(
-            rop.CALL, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [box1]
+            rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [box1]
         )
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
 
     def test_call_doesnt_invalidate_unescaped_boxes(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         h.new(box1)
         assert h.is_unescaped(box1)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.CALL,
+        h.invalidate_caches(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -557,10 +681,13 @@ class TestHeapCache(object):
 
     def test_call_doesnt_invalidate_unescaped_array_boxes(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box3 = RefFrontendOp(3)
+        lengthbox1 = IntFrontendOp(11)
         h.new_array(box1, lengthbox1)
         assert h.is_unescaped(box1)
         h.setarrayitem(box1, index1, box3, descr1)
-        h.invalidate_caches(rop.CALL,
+        h.invalidate_caches(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -568,6 +695,8 @@ class TestHeapCache(object):
 
     def test_bug_missing_ignored_operations(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
@@ -590,18 +719,22 @@ class TestHeapCache(object):
         # calling some residual code that changes the values on box3: then
         # the content of box2 is still cached at the old value.
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
         h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
         assert h.getfield(box1, descr1) is box2
-        h.invalidate_caches(rop.CALL_MAY_FORCE, None, [])
+        h.invalidate_caches(rop.CALL_MAY_FORCE_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is None
 
     def test_bug_heap_cache_is_cleared_but_not_is_unescaped_2(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
+        box2 = RefFrontendOp(2)
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
@@ -617,13 +750,14 @@ class TestHeapCache(object):
             EF_ELIDABLE_OR_MEMORYERROR = 3
             EF_ELIDABLE_CAN_RAISE = 4
         descr.get_extra_info = XTra
-        h.invalidate_caches(rop.CALL, descr, [])
+        h.invalidate_caches(rop.CALL_N, descr, [])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is box2
 
     def test_is_likely_virtual(self):
         h = HeapCache()
+        box1 = RefFrontendOp(1)
         h.new(box1)
         assert h.is_unescaped(box1)
         assert h.is_likely_virtual(box1)
@@ -633,3 +767,29 @@ class TestHeapCache(object):
         h._escape_box(box1)
         assert not h.is_unescaped(box1)
         assert not h.is_likely_virtual(box1)
+
+    def test_is_likely_virtual_2(self):
+        h = HeapCache()
+        box1 = RefFrontendOp(1)
+        h.new(box1)
+        assert h.is_unescaped(box1)
+        assert h.is_likely_virtual(box1)
+        h.reset_keep_likely_virtuals()
+        assert not h.is_unescaped(box1)
+        assert h.is_likely_virtual(box1)
+        h.reset()     # reset everything
+        assert not h.is_unescaped(box1)
+        assert not h.is_likely_virtual(box1)
+
+    def test_is_likely_virtual_3(self):
+        h = HeapCache()
+        box1 = RefFrontendOp(1)
+        h.new(box1)
+        assert h.is_unescaped(box1)
+        assert h.is_likely_virtual(box1)
+        h.reset_keep_likely_virtuals()
+        assert not h.is_unescaped(box1)
+        assert h.is_likely_virtual(box1)
+        h.class_now_known(box1)     # interaction of the two families of flags
+        assert not h.is_unescaped(box1)
+        assert h.is_likely_virtual(box1)
