@@ -239,7 +239,8 @@ class WarmRunnerDesc(object):
         elif self.opt.listops:
             self.prejit_optimizations_minimal_inline(policy, graphs)
 
-        self.build_meta_interp(ProfilerClass)
+        self.build_meta_interp(ProfilerClass,
+                             translator.config.translation.jit_opencoder_model)
         self.make_args_specifications()
         #
         from rpython.jit.metainterp.virtualref import VirtualRefInfo
@@ -464,7 +465,7 @@ class WarmRunnerDesc(object):
         if no_stats:
             stats = history.NoStats()
         else:
-            stats = history.Stats()
+            stats = history.Stats(None)
         self.stats = stats
         if translate_support_code:
             self.annhelper = MixLevelHelperAnnotator(self.translator.rtyper)
@@ -478,11 +479,17 @@ class WarmRunnerDesc(object):
             cpu.supports_singlefloats = False
         self.cpu = cpu
 
-    def build_meta_interp(self, ProfilerClass):
+    def build_meta_interp(self, ProfilerClass, opencoder_model):
+        from rpython.jit.metainterp.opencoder import Model, BigModel
         self.metainterp_sd = MetaInterpStaticData(self.cpu,
                                                   self.opt,
                                                   ProfilerClass=ProfilerClass,
                                                   warmrunnerdesc=self)
+        if opencoder_model == 'big':
+            self.metainterp_sd.opencoder_model = BigModel
+        else:
+            self.metainterp_sd.opencoder_model = Model            
+        self.stats.metainterp_sd = self.metainterp_sd
 
     def make_virtualizable_infos(self):
         vinfos = {}
