@@ -22,9 +22,6 @@ import os
 import sys
 import ctypes.util
 
-from rpython.tool.ansi_print import ansi_log
-log = py.log.Producer("libffi")
-py.log.setconsumer("libffi", ansi_log)
 
 # maaaybe isinstance here would be better. Think
 _MSVC = platform.name == "msvc"
@@ -594,9 +591,12 @@ class FuncPtr(AbstractFuncPtr):
                                             intmask(argtypes[i].c_size),
                                             flavor='raw')
         if restype != ffi_type_void:
-            size = adjust_return_size(intmask(restype.c_size))
+            self.restype_size = intmask(restype.c_size)
+            size = adjust_return_size(self.restype_size)
             self.ll_result = lltype.malloc(rffi.VOIDP.TO, size,
                                            flavor='raw')
+        else:
+            self.restype_size = -1
 
     def push_arg(self, value):
         #if self.pushed_args == self.argnum:
@@ -633,7 +633,12 @@ class FuncPtr(AbstractFuncPtr):
                             rffi.cast(VOIDPP, self.ll_args))
         if RES_TP is not lltype.Void:
             TP = lltype.Ptr(rffi.CArray(RES_TP))
-            res = rffi.cast(TP, self.ll_result)[0]
+            ptr = self.ll_result
+            if _BIG_ENDIAN and RES_TP in TYPE_MAP_INT:
+                # we get a 8 byte value in big endian
+                n = rffi.sizeof(lltype.Signed) - self.restype_size
+                ptr = rffi.ptradd(ptr, n)
+            res = rffi.cast(TP, ptr)[0]
         else:
             res = None
         self._clean_args()
