@@ -1,5 +1,4 @@
 from rpython.rtyper.lltypesystem.lltype import DelayedPointer
-from rpython.translator.simplify import get_graph
 from rpython.tool.algo.unionfind import UnionFind
 
 
@@ -55,11 +54,7 @@ class GraphAnalyzer(object):
     def analyze_startblock(self, block, seen=None):
         return self.bottom_result()
 
-    def analyze_external_call(self, op, seen=None):
-        try:
-            funcobj = op.args[0].value._obj
-        except DelayedPointer:
-            return self.bottom_result()
+    def analyze_external_call(self, funcobj, seen=None):
         result = self.bottom_result()
         if hasattr(funcobj, '_callbacks'):
             bk = self.translator.annotator.bookkeeper
@@ -80,12 +75,24 @@ class GraphAnalyzer(object):
 
     def analyze(self, op, seen=None, graphinfo=None):
         if op.opname == "direct_call":
-            graph = get_graph(op.args[0], self.translator)
-            if graph is None:
-                x = self.analyze_external_call(op, seen)
+            try:
+                funcobj = op.args[0].value._obj
+            except DelayedPointer:
+                return self.top_result()
+            if funcobj is None:
+                # We encountered a null pointer.  Calling it will crash.
+                # However, the call could be on a dead path, so we return the
+                # bottom result here.
+                return self.bottom_result()
+            if getattr(funcobj, 'external', None) is not None:
+                x = self.analyze_external_call(funcobj, seen)
                 if self.verbose and x:
                     self.dump_info('analyze_external_call %s: %r' % (op, x))
                 return x
+            try:
+                graph = funcobj.graph
+            except AttributeError:
+                return self.top_result()
             x = self.analyze_direct_call(graph, seen)
             if self.verbose and x:
                 self.dump_info('analyze_direct_call(%s): %r' % (graph, x))
