@@ -8,6 +8,6 @@
 #define PYPY_MAIN_FUNCTION main
 #endif
 
-RPY_EXTERN char *RPython_StartupCode(void);
+RPY_EXTERN void RPython_StartupCode(void);
 RPY_EXPORTED int PYPY_MAIN_FUNCTION(int argc, char *argv[]);
 #endif  /* PYPY_STANDALONE */
